'''This module implements parsing of the description file
'''

import xml.sax.handler

import finalconfig.section
import finalconfig.entry

class ContentHandler (xml.sax.handler.ContentHandler):
    '''Implementation via SAX parser.'''
    
    def __init__ (self, session):
        '''Initialise with the sission, that will receive the description data.
    
        session must implement the following functions:
        add_section (section)
        '''
        self.session = session
        self.section_stack = []
        self.entry_stack = []
        self.text = ''

    def startElement (self, name, attr):
        if name=='section':
            self._add_section (attr)
        elif name=='entry':
            entry = self._make_entry (attr)
            if len (self.entry_stack)!=0:
                parent = self.entry_stack[-1]
                if issubclass (parent.__class__, finalconfig.entry.ContainerEntry):
                    parent.sub.append (entry)
            elif len (self.section_stack)!=0:
                parent = self.section_stack[-1]
                parent.sub.append (entry)
            self.entry_stack.append (entry)
        elif name=='value': # default value
            pass
        elif name=='title':
            pass
        elif name=='hint':
            pass
        elif name=='handle':
            pass
        elif name=='min':
            pass
        elif name=='max':
            pass
        elif name=='digits':
            pass
        
    def endElement (self, name):
        if self.text != '':
            text = ' '.join (self.text.split())
            self.text = ''
        if name=='section':
            self.section_stack.pop()
        elif name=='entry':            
            self.entry_stack.pop()
        elif name=='value': # default value
            if len (self.entry_stack) != 0:
                self.entry_stack[-1].set_default (text)
        elif name=='title':
            if len (self.entry_stack) != 0:
                self.entry_stack[-1].set_title (text)
            elif len (self.section_stack) != 0:
                self.section_stack[-1].set_title (text)
        elif name=='hint':
            if len (self.entry_stack) != 0:
                self.entry_stack[-1].set_tooltip (text)       
        elif name=='handle':
            if len (self.entry_stack) != 0:
                entry = self.entry_stack[-1]
                if issubclass (entry.__class__, finalconfig.entry.ListStringEntry):
                    entry.set_handle (text)
        elif name=='min':
            if len (self.entry_stack) != 0:
                entry = self.entry_stack[-1]
                if issubclass (entry.__class__, finalconfig.entry.NumericEntry):
                    entry.set_min (float (text))            
        elif name=='max':
            if len (self.entry_stack) != 0:
                entry = self.entry_stack[-1]
                if issubclass (entry.__class__, finalconfig.entry.NumericEntry):
                    entry.set_max (float (text))            
        elif name=='digits':
            if len (self.entry_stack) != 0:
                entry = self.entry_stack[-1]
                if issubclass (entry.__class__, finalconfig.entry.NumericEntry):
                    entry.set_digits (int (text))            
                
    def characters (self, content):
        self.text = self.text + content

    def _add_section (self, attr):
        parent = None
        if len (self.section_stack)!=0:
            parent = self.section_stack[-1]
        hidden = False
        if parent is not None and parent.hidden is True:
            hidden = True
        elif attr.get ('hidden')=='true':
            hidden = True
        section = finalconfig.section.Section (parent, hidden)
        if not hidden:
            self.session.add_section (section)
        self.section_stack.append (section)

    def _make_entry (self, attr):
        type_ = attr.get ('type')
        if type_=='group':
            return finalconfig.entry.GroupEntry()
        elif type_=='bool':
            return finalconfig.entry.BoolEntry (attr.get ('id'))
        elif type_=='string':
            return finalconfig.entry.StringEntry (attr.get ('id'))
        elif type_=='list':
            return finalconfig.entry.ListEntry (attr.get ('id'))
        elif type_=='liststring':
            return finalconfig.entry.ListStringEntry()
        elif type_=='numeric':
            exact = False
            if 'exact' in attr.keys() and attr.get ('exact')=='true':
                exact = True
            return finalconfig.entry.NumericEntry (attr.get ('id'), exact)            

